from enum import Enum
from typing import Optional
from pydantic import BaseModel

from happyrobot.models.call import Call
from happyrobot.models.message import Message


class StartContent(BaseModel):
    recording: Optional[str] = None


class EndContent(BaseModel):
    messages: list[Message]
    tools: Optional[dict] = None



class EventType(str, Enum):
    Start = "start"
    Message = "message"
    End = "end"


class Event(BaseModel):
    type: EventType
    call: Call
    content: dict

    class Config:
        use_enum_values = True


import datetime
import pytz


def parse_date(timestamp_str: str):
    # Parse the timestamp string into a datetime object
    timestamp = datetime.datetime.fromisoformat(timestamp_str)

    # Convert the timestamp to Eastern Standard Time (EST)
    est_timezone = pytz.timezone('America/New_York')
    timestamp_est = timestamp.astimezone(est_timezone)

    # Format the timestamp into a more readable format including the timezone
    readable_format_with_timezone = timestamp_est.strftime("%B %d, %Y %H:%M:%S %Z")

    return readable_format_with_timezone



from typing import Optional
from pydantic import BaseModel, Field
from typing import Optional


class CallMetadata(BaseModel):
    call_sid: Optional[str] = None
    ffrom: str = Field(alias="from")
    to: str


class Call(BaseModel):
    id: str
    organization_id: str
    metadata: CallMetadata
    created: str
    


from copy import deepcopy


def combine_messages(messages: list[dict]) -> list[dict]:
    new_messages: list[dict] = []
    tracking_message: dict = None

    for message in messages:
        if tracking_message:
            # If the new message has the same role as the tracking,
            # add the content and continue
            if message["role"] == tracking_message["role"]:
                tracking_message["content"] += " " + message["content"]
                if "tool_calls" in tracking_message:
                    if tracking_message["tool_calls"] is None:
                        tracking_message["tool_calls"] = []
                    assert tracking_message["tool_calls"] is not None, message
                    tool_calls = message.get("tool_calls", [])
                    if tool_calls is not None:
                        tracking_message["tool_calls"].extend(tool_calls)
                elif "tool_calls" in message:
                    tracking_message["tool_calls"] = message["tool_calls"]
                continue

            # Otherwise, add the traking to the list of messages
            # cause the sender changed
            new_messages.append(tracking_message)

        # Set this message as the tracking message
        tracking_message = deepcopy(message)

    # Adding the last message
    if tracking_message:
        new_messages.append(tracking_message)

    return new_messages




from setuptools import setup, find_packages


with open('README.md') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()


def read_requirements(file):
    with open(file) as f:
        return f.read().splitlines()

requirements = read_requirements('requirements.txt')

VERSION = '0.2.5'

setup(
    author="Happyrobot",
    author_email='founders@happyrobot.ai',
    name='happyrobot',
    python_requires='>=3.9',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    description="This package lets you interact with Happyrobot directly from Python.",
    install_requires=requirements,
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='happyrobot',
    packages=find_packages(include=['happyrobot', 'happyrobot.*']),
    version=VERSION,
    zip_safe=False,
)



from enum import Enum
from typing import Optional, Literal
from pydantic import BaseModel
import json
from typing import List, Dict, Any

from happyrobot.src.utils.message_utils import combine_messages


class MessageRole(str, Enum):
    System = "system"
    User = "user"
    Assistant = "assistant"
    Tool = "tool"


class Function(BaseModel):
    name: str
    arguments: str
    

class ToolCall(BaseModel):
    type: Literal["function"] = "function"
    id: str
    function: Function
    


class Message(BaseModel):
    id: str
    role: MessageRole
    content: str
    tool_calls: Optional[list[ToolCall]] = None
    tool_call_id: Optional[str] = None
    name: Optional[str] = None
    


class Messages(BaseModel):
    messages: list[Message]
    
    def _process_tool_call(self, tool_call: ToolCall):
        # Helper function to process each tool call, including nested 'function' key
        processed_call = {}
        for key, value in tool_call.model_dump().items():
            if key == "function":
                # Process the 'function' key's value, which is expected to be a dict
                value = {k: v for k, v in value.items() if not (k == "arguments" and v.strip() == "{}")}
            elif key == "id":
                continue
            processed_call[key] = value
        return processed_call
    
    def to_dict(self, drop_id: bool = True):
        messages = [
            {
                **({"id": message.id} if not drop_id else {}),
                "role": message.role.value,
                **({"name": message.name} if message.name else {}),
                "content": message.content.strip() if message.content else None,
                **({"tool_calls": [self._process_tool_call(tc) for tc in message.tool_calls]} if message.tool_calls else {}),
            }
            for message in self.messages if message.content.strip() != ""
        ]
        
        # Remove system messages
        messages = [message for message in messages if message["role"] != "system"]
        # Concatenate messages just in case
        messages = combine_messages(messages)
        return messages
    
    @staticmethod
    def serialize(messages: List[Dict[str, Any]]):
        def is_json_string(s):
            # Helper function to check if a string is JSON
            try:
                json_object = json.loads(s)
                return isinstance(json_object, dict)  # Check if the loaded object is a dictionary
            except json.JSONDecodeError:
                return False

        def serialize_item(item):
            # Recursively serialize each item, checking for JSON strings
            if isinstance(item, dict):
                return {key: serialize_item(value) for key, value in item.items()}
            elif isinstance(item, list):
                return [serialize_item(element) for element in item]
            elif isinstance(item, str) and is_json_string(item):
                return json.loads(item)  # Parse the JSON string into a dict
            else:
                return item  # Return the item as is if it's not a dict, list, or JSON string

        return json.dumps([serialize_item(message) for message in messages])
        
